<?php

namespace Litecms\Institute\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Litecms\Institute\Http\Requests\InstituteRequest;
use Litecms\Institute\Interfaces\InstituteRepositoryInterface;
use Litecms\Institute\Models\Institute;

/**
 * Resource controller class for institute.
 */
class InstituteResourceController extends BaseController
{

    /**
     * Initialize institute resource controller.
     *
     * @param type InstituteRepositoryInterface $institute
     *
     * @return null
     */
    public function __construct(InstituteRepositoryInterface $institute)
    {
        parent::__construct();
        $this->repository = $institute;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Litecms\Institute\Repositories\Criteria\InstituteResourceCriteria::class);
    }

    /**
     * Display a list of institute.
     *
     * @return Response
     */
    public function index(InstituteRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Litecms\Institute\Repositories\Presenter\InstitutePresenter::class)
                ->$function();
        }

        $institutes = $this->repository->paginate();

        return $this->response->title(trans('institute::institute.names'))
            ->view('institute::institute.index', true)
            ->data(compact('institutes'))
            ->output();
    }

    /**
     * Display institute.
     *
     * @param Request $request
     * @param Model   $institute
     *
     * @return Response
     */
    public function show(InstituteRequest $request, Institute $institute)
    {

        if ($institute->exists) {
            $view = 'institute::institute.show';
        } else {
            $view = 'institute::institute.new';
        }

        return $this->response->title(trans('app.view') . ' ' . trans('institute::institute.name'))
            ->data(compact('institute'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new institute.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(InstituteRequest $request)
    {

        $institute = $this->repository->newInstance([]);
        return $this->response->title(trans('app.new') . ' ' . trans('institute::institute.name')) 
            ->view('institute::institute.create', true) 
            ->data(compact('institute'))
            ->output();
    }

    /**
     * Create new institute.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(InstituteRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $institute                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('institute::institute.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('institute/institute/' . $institute->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/institute/institute'))
                ->redirect();
        }

    }

    /**
     * Show institute for editing.
     *
     * @param Request $request
     * @param Model   $institute
     *
     * @return Response
     */
    public function edit(InstituteRequest $request, Institute $institute)
    {
        return $this->response->title(trans('app.edit') . ' ' . trans('institute::institute.name'))
            ->view('institute::institute.edit', true)
            ->data(compact('institute'))
            ->output();
    }

    /**
     * Update the institute.
     *
     * @param Request $request
     * @param Model   $institute
     *
     * @return Response
     */
    public function update(InstituteRequest $request, Institute $institute)
    {
        try {
            $attributes = $request->all();

            $institute->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('institute::institute.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('institute/institute/' . $institute->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('institute/institute/' . $institute->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the institute.
     *
     * @param Model   $institute
     *
     * @return Response
     */
    public function destroy(InstituteRequest $request, Institute $institute)
    {
        try {

            $institute->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('institute::institute.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('institute/institute/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('institute/institute/' . $institute->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple institute.
     *
     * @param Model   $institute
     *
     * @return Response
     */
    public function delete(InstituteRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('institute::institute.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('institute/institute'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/institute/institute'))
                ->redirect();
        }

    }

    /**
     * Restore deleted institutes.
     *
     * @param Model   $institute
     *
     * @return Response
     */
    public function restore(InstituteRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('institute::institute.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/institute/institute'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/institute/institute/'))
                ->redirect();
        }

    }

}
